% @SameDifferent/StateMatrixSection
% BWB, July 2008

% [x, y] = StateMatrixSection(obj, action, x, y)
%
% HELP HERE
%
% PARAMETERS:
% -----------
%
% obj      Default object argument.
%
% action   One of:
%            'init'         To initialise the section
%
%            'next_trial'   To set up the state matrix for the next trial
%
%            'reinit'       Delete all of this section's GUIs and data,
%                           and reinit, at the same position on the same
%                           figure as the original section GUI was placed.
%
% x, y     Relevant to action = 'init'; they indicate the initial
%          position to place the GUI at, in the current figure window
%
% RETURNS:
% --------
%
% [x, y]   When action == 'init', returns x and y, pixel positions on
%          the current figure, updated after placing of this section's GUI. 
%


function  [] =  StateMatrixSection(obj, action)

% global left1led;
% global center1led;
% global right1led;
% global left1water;
% global right1water;


GetSoloFunctionArgs;


switch action
  case 'init', 
    
    feval(mfilename, obj, 'next_trial');

%% next_trial
  case 'next_trial',

    sma = StateMachineAssembler('full_trial_structure');
    
    min_time= 1E-4;  % This is close to the minumum time allowed for a state transition.
    
    left1led           = bSettings('get', 'DIOLINES', 'left1led'); 
    center1led         = bSettings('get', 'DIOLINES', 'center1led');
    right1led          = bSettings('get', 'DIOLINES', 'right1led'); 
    left1water         = bSettings('get', 'DIOLINES', 'left1water'); 
    right1water        = bSettings('get', 'DIOLINES', 'right1water'); 
    align_bit          = bSettings('get', 'DIOLINES', 'align_bit'); 
    if isnan(align_bit), align_bit=0;end
    
%% set up sounds
    switch value(SoundSource),
        case 0, % if using sound table
            % choose the correct sounds to play; if SoundsOn == 0, 
            % stimulus_id == 0
            k = SidesSection(obj, 'get_current_sound');
            try
                s1_id  = SoundsOn*SoundManagerSection(obj, 'get_sound_id', sprintf([value(tablename) 'Sound%dS1'], k));
                s1_dur = SoundManagerSection(obj, 'get_sound_duration', sprintf([value(tablename) 'Sound%dS1'], k));
            catch
                s1_id  = 0;
                s1_dur = 1;
            end;

            try
                s2_id  = SoundsOn*SoundManagerSection(obj, 'get_sound_id', sprintf([value(tablename) 'Sound%dS2'], k));
                s2_dur = SoundManagerSection(obj, 'get_sound_duration', sprintf([value(tablename) 'Sound%dS2'], k));
            catch
                s2_id  = 0;
                s2_dur = 1;
            end;
        case 1, % if using pbups
            s1_id  = SoundsOn*SoundManagerSection(obj, 'get_sound_id', 'PBupsSound');
            s1_dur = SoundManagerSection(obj, 'get_sound_duration', 'PBupsSound');
            s2_id  = 0;
            s2_dur = 0;
    end;
    
%% set up reward quantity and timing
    [LeftWValveTime RightWValveTime] = WaterValvesSection(obj, 'get_water_times', hit_streak);
    [L_baited,R_baited] = ReinforcementSection(obj,'baited');
        RewardSound = ReinforcementSection(obj,'get_sound_id');
        
        if L_baited==0
            LeftWValveTime=0;
        end
        
        if R_baited==0
            RightWValveTime=0;
        end
    if SoundSource == 0,
        try % water multiplier specific to this trial type
            type_wtr_mul = SoundTableSection(obj, 'get', value(tablename), k, 'wtr_mul');
        catch
            type_wtr_mul = 1;
        end;
    else
        type_wtr_mul = 1;
    end;

%% set up correct responses
    if SidesSection(obj, 'get_current_side') == 'l',
        if strcmp(value(reward_timing), 'from spoke'),
            LeftResponseState = 'left_reward';
        elseif strcmp(value(reward_timing), 'from cpoke'),
            LeftResponseState = 'hit_state';
        else
            warning('StateMatrixSection:Invalid', 'doesn''t know reward timing!');
            return;
        end;
        
        if PunishSideChoice == 1, 
            if SideChoicePunishmentType == 1,
                % punishment: temporary white noise
                RightResponseState = 'temporary_punishment';
            else
                % punishment: trial terminates
                RightResponseState = 'error_state1';
            end;
        else
            RightResponseState = 'current_state';
        end;
        
        % set up which side LED to turn on
        switch value(side_lights),
          case 'correct side only',     side_led = left1led;
          case 'both sides on',         side_led = left1led+right1led;
          case 'both sides off',        side_led = 0;
          case 'anticorrect side only', side_led = right1led;
          case 'on during cpokes',      side_led = 0;  % side_led is who to turn on after cpoke
          otherwise
            warning('SameDifferent:UnknownSetting', 'Don''t know what to do with the side_lights setting');
        end;
        correct_side_led   = left1led; %#ok<NASGU>
        correct_side_water = left1water;
        correct_side_valvetime = LeftWValveTime*type_wtr_mul*left_wtr_mult;
    else
        if strcmp(value(reward_timing), 'from spoke'),
            RightResponseState = 'right_reward';
        elseif strcmp(value(reward_timing), 'from cpoke'),
            RightResponseState = 'hit_state';
        else
            warning('StateMatrixSection:Invalid', 'doesn''t know reward timing!');
            return;
        end;

        if PunishSideChoice == 1, 
            if SideChoicePunishmentType == 1,
                % punishment: temporary white noise    
                LeftResponseState = 'temporary_punishment';
            else
                % punishment: trial terminates
                LeftResponseState = 'error_state1';
            end;
        else
            LeftResponseState = 'current_state';
        end;
          
        % set up which side LED to turn on
        switch value(side_lights),
          case 'correct side only',     side_led = right1led;
          case 'both sides on',         side_led = right1led+left1led;
          case 'both sides off',        side_led = 0;
          case 'anticorrect side only', side_led = left1led;
          case 'on during cpokes',      side_led = 0;  % side_led is who to turn on after cpoke
          otherwise
            warning('SameDifferent:UnknownSetting', 'Don''t know what to do with the side_lights setting');
        end;
        correct_side_led   = right1led; %#ok<NASGU>
        correct_side_water = right1water;
        correct_side_valvetime = RightWValveTime*type_wtr_mul*right_wtr_mult;
    end;


    sma = add_scheduled_wave(sma, 'name', 'direct_reward', 'preamble', reward_delay(1), ...
        'sustain', correct_side_valvetime, 'DOut', correct_side_water);

   

%% set up n_center_pokes
    if count_cpokes == 1,
        if n_center_pokes == 0,
            first_state  = 'wait_for_spoke';
            after_cpoke1 = 'wait_for_spoke';
        elseif n_center_pokes == 1,
            first_state  = 'wait_for_cpoke1';
            after_cpoke1 = 'center_2_side_gap';
        else
            first_state  = 'wait_for_cpoke1';
            after_cpoke1 = 'cpoke1';
        end;
    else % if we're not counting cpokes, then there's only 1 cpoke state
        first_state  = 'wait_for_cpoke1';
        after_cpoke1 = 'cpoke1';
    end;

    % cpoke_timer is the least amount of time the rat has to hold his nose
    % in the center to advance to the next cpoke state.  It is at least
    % 10 msec.
    cpoke_timer = max(nose_in_center/value(n_center_pokes), 0.01);
    
    % if n_center_pokes > 1, we're going to make n_center_pokes states that
    % each last for cpoke_timer sec duration.  If the rat does a Cout, we
    % move onto the next wait_for_cpoke; if the rat is still in the poke
    % for the duration of cpoke_timer, we automatically proceed to the next
    % cpoke state

    if strcmp(value(center_light), 'on during cpokes'),
        cpoke_led = center1led;
    else
        cpoke_led = 0;
    end;
    
    if strcmp(side_lights, 'on during cpokes'),
       cpoke_led = cpoke_led + left1led + right1led;
    end;
    
%% clean up sounds
snd_list=SoundManagerSection(SameDifferent, 'get_sound_id','all');
sma = add_multi_sounds_state(sma, -1*snd_list,'state_name','clean_sound_state','return_state','iti');
    
%% ITI
    
    if PunishITIPokes == 1, iti_poke_state = 'iti_pun';
    else                    iti_poke_state = 'current_state';
    end;
    
     
    ITI = PenaltySection(obj, 'get_ITI');
    if ~isempty(hit_history) && hit_history(n_done_trials)==0, 
		this_ITI = ITI + miss_ITI;
	else
		this_ITI = ITI;
	end;
	
    sma = add_state(sma, 'name', 'ITI', 'self_timer', this_ITI, ...
        'input_to_statechange', {'Tup', first_state; ...
                                 'Cin', iti_poke_state; ...
                                 'Lin', iti_poke_state; ...
                                 'Rin', iti_poke_state; ...
                                 });
                             

    sma = PunishInterface(obj, 'add_sma_states', 'ITIPun', sma, ...
        'name', 'iti_pun', ...
        'exitstate', 'iti');

    
    
%% wait for center poke 1    
    nic_sound_id=SoundManagerSection(obj, 'get_sound_id', 'nic_sound');
    
     if PunishCL1BadPokes == 1, poke_state = 'cl1_pun';
     else                       poke_state = 'current_state';
     end;
     
     % center poke schedule:
     if cpoke_schedule == 1, % if determined center pokes
         center_poke_state = 'play_stimulus_sound';
     else % if free center pokes
         center_poke_state = 'center_in_with_sounds';
     end;
         
    
     if count_cpokes == 1,
       sma = add_state(sma, 'name', 'WAIT_FOR_CPOKE1', ...
         'output_actions', {'DOut', center1led}, ...
         'input_to_statechange', {'Cin', center_poke_state; ...
                                  'Lin', poke_state; ...
                                  'Rin', poke_state; ...
                                 });
     else
       sma = add_state(sma, 'name', 'WAIT_FOR_CPOKE1', 'self_timer', wait_for_cpoke1_tout, ...
         'output_actions', {'DOut', center1led}, ...
         'input_to_statechange', {'Tup', 'center_2_side_gap' ; ...
                                  'Cin', center_poke_state; ...
                                  'Lin', poke_state; ...
                                  'Rin', poke_state; ...
                                 });
     end;

     sma = add_state(sma, 'name', 'cl1_pun', 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -nic_sound_id}, ...
        'input_to_statechange', {'Tup', 'cl1_pun_pi'});   
     
     sma = PunishInterface(obj, 'add_sma_states', 'CenterLightPun', sma, ...
         'name', 'cl1_pun_pi', ...
         'exitstate', 'wait_for_cpoke1');


    

%% set up stimuli
    if s1_id == 0, % if there's no sound to play
        waves_on = 'empty_timer';
        waves_off = '-empty_timer';
        sma = add_scheduled_wave(sma, 'name', waves_on, 'preamble', 1);
    else
        if SoundSource == 0,
            [gap1 gap2] = SoundTableSection(obj, 'get_gap_durations', value(tablename));
        else % if using pbups
            gap1 = 0;
            gap2 = 0;
        end;
        cycle_length = s1_dur + gap1;
        
        two_sounds=SoundTableSection(obj, 'get_tableparam', value(tablename),'two_sounds');
        
        if two_sounds && s2_id > 0,
            cycle_length = cycle_length + s2_dur + gap2;
        end;

        % if From Stim Start, make as many scheduled waves to trigger sounds as
        % n_stimulus_cycles, up to 10 (because we're only allowed 32 scheduled
        % waves total
        % if From Answer Poke or During Cpoke Only, make 10 scheduled wave -- 
        % hopefully that will take care of most scenarios
        if cpoke_schedule == 0,  % if the rat is free grazing for sound
            cycles = 10;
            part_cycle = 0;
        elseif strcmp(stim_timing, 'From Stim Start'),
            cycles = min(10, n_stimulus_cycles);
            part_cycle = 0;
        elseif strcmp(stim_timing, 'Fixed T After Start'),
            cycles = min(10, floor(fixed_stim_dur/cycle_length)); % # of whole cycles
            part_cycle = fixed_stim_dur - cycles*cycle_length;   % duration of part_cycle; may be 0
         %  if part_cycle > 1,        part_cycle = 1; end; % JCE - This
         %  line seems logically false. <~>
            if part_cycle < min_time, part_cycle = 0; end;
            if part_cycle > 0 && cycles==10, cycles = 9; end;  % only have 10 sched waves total
        else
            cycles = 10;
            part_cycle = 0;
        end;

        waves_on  = '';
        waves_off = '';
        for i = 1:cycles,
            wavename = sprintf('s1_cycle_%d', i);
            waves_on  = [waves_on  '+' wavename]; %#ok<AGROW>
            waves_off = [waves_off '-' wavename]; %#ok<AGROW>
            sma = add_scheduled_wave(sma, 'name', wavename, 'preamble', stim_start_delay+cycle_length*(i-1), 'sustain', s1_dur, 'sound_trig', s1_id);
        end;
        if s2_id > 0,
            for i = 1:cycles,
                wavename = sprintf('s2_cycle_%d', i);
                waves_on  = [waves_on   '+' wavename]; %#ok<AGROW>
                waves_off = [waves_off  '-' wavename]; %#ok<AGROW>
                sma = add_scheduled_wave(sma, 'name', wavename, 'preamble', stim_start_delay+cycle_length*(i-1)+s1_dur+gap1, 'sustain', s2_dur, 'sound_trig', s2_id);                
            end;
        end;
        if part_cycle > 0,  % If we have *part* of a full cycle that we need to play, add that bit.
          i = cycles+1;
          wavename = sprintf('s1_cycle_%d', i);
          waves_on  = [waves_on  '+' wavename];
          waves_off = [waves_off '-' wavename];
          sma = add_scheduled_wave(sma, 'name', wavename, 'preamble', stim_start_delay+cycle_length*(i-1), 'sustain', min(s1_dur, part_cycle), 'sound_trig', s1_id);
          if s2_id > 0 && part_cycle > s1_dur+gap1,
            wavename = sprintf('s2_cycle_%d', i);
            waves_on  = [waves_on   '+' wavename];
            waves_off = [waves_off  '-' wavename];
            sma = add_scheduled_wave(sma, 'name', wavename, 'preamble', stim_start_delay+cycle_length*(i-1)+s1_dur+gap1, 'sustain', min(s2_dur, part_cycle-s1_dur-gap1), 'sound_trig', s2_id);
          end;
        end;
    end;
    
    % PLAY_STIMULUS_SOUND: first make sure the stimulus sound and the
    % scheduled waves are off
    sma = add_state(sma, 'name', 'play_stimulus_sound', 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s1_id}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s2_id}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', waves_off}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    
    if strcmp(value(reward_timing), 'from cpoke'),
        sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'SchedWaveTrig', 'direct_reward'}, ...
            'input_to_statechange', {'Tup', 'current_state+1'});
    end
    
    % then set all sounds playing by triggering all the scheduled waves,
    % and jump to the state appropriate for immediately after the first
    % cpoke:
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', waves_on}, ...
        'input_to_statechange', {'Tup', after_cpoke1});


    % PLAY_STIMULUS_GOTO_C2SGAP: Just like play_stimulus_sound, except that
    % after this sequence, we go to the center_2_side_gap state
    sma = add_state(sma, 'name', 'PLAY_STIMULUS_GOTO_C2SGAP', 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s1_id}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s2_id}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', waves_off}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    
    if strcmp(value(reward_timing), 'from cpoke'),
        sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'SchedWaveTrig', 'direct_reward'}, ...
            'input_to_statechange', {'Tup', 'current_state+1'});
    end
    
    % then set all sounds playing by triggering all the scheduled waves
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', waves_on}, ...
        'input_to_statechange', {'Tup', 'center_2_side_gap'});
      

    
%% set up cpokes1-5


     
    if PunishCL1BadPokes == 1, poke_state = 'cl1_pun';
    else                       poke_state = 'current_state';
    end;

    if count_cpokes == 1,  % if we're counting cpokes
        sma = add_state(sma, 'name', 'CPOKE1', 'self_timer', cpoke_timer, ...
            'output_actions', {'DOut', cpoke_led+align_bit}, ...
            'input_to_statechange', {'Cout', 'wait_for_cpoke2'; ...
                                     'Tup',  'cpoke2'});

        for i = 2:5,
            waitstate_name = sprintf('wait_for_cpoke%d', i);
            holdstate_name = sprintf('cpoke%d', i);

            if i < n_center_pokes,
                cinstate_name = holdstate_name;
                tupstate_name = sprintf('cpoke%d', i+1);
                nxtstate_name = sprintf('wait_for_cpoke%d', i+1);
            else
                cinstate_name = 'center_2_side_gap';
                tupstate_name = 'center_2_side_gap';
                nxtstate_name = 'current_state';
            end;

            sma = add_state(sma, 'name', waitstate_name, ...
                    'self_timer', wait_for_cpoke_grace, ...
                    'output_actions', {'DOut', cpoke_led}, ...
                    'input_to_statechange', {'Cin', cinstate_name; ...
                                            'Lin', poke_state; ...
                                            'Rin', poke_state; ...
                                            'Tup', 'violation_state'} ...
                    );

            sma = add_state(sma, 'name', holdstate_name, 'self_timer', cpoke_timer, ...
                    'output_actions', {'DOut', cpoke_led}, ...
                    'input_to_statechange', {'Cout', nxtstate_name; ...
                                             'Tup',  tupstate_name} ...
                    );  
        end;
    else  % if we're not counting cpokes, set up one cpoke1 state that allows excursions 
        % from the cpoke of duration < legal_cbreak.
        
        % in the "one cpoke w/legal breaks" mode, we can choose whether to
        % respond to a violation by ending the trial or by a temp
        % violation that returns to wait_for_cpoke1
        
        if PunishCL1BadPokes == 1, poke_state = 'bad_side_poke';
        else                       poke_state = 'current_state';
        end;

        if temp_pun_on_violation,   violation_goto = 'temp_violation';
        else                        violation_goto = 'violation_state';
        end;
                
     
    
        if WaitForCenterNoseOut==1, 
            nic_end_goto = 'wait_for_center_nose_out';
           
            
        else
            nic_end_goto = 'center_2_side_gap';
           
        end;
        
        sma = add_scheduled_wave(sma, 'name', 'cpoke_timer',  'preamble', nose_in_center);
        sma = add_scheduled_wave(sma, 'name', 'legal_cbreak', 'preamble', legal_cbreak);

        % A state that turns sched waves off and goes to a violation state:
        sma = add_state(sma, 'name', 'BAD_SIDE_POKE', 'self_timer', min_time, ...
          'output_actions', {'SchedWaveTrig', '-cpoke_timer-legal_cbreak'}, ...
          'input_to_statechange', {'Tup', 'cl1_pun'});

        % trigger the start of the timer:
        
        if nic_sound==1
            
            sma = add_state(sma, 'name', 'CPOKE1', 'self_timer', min_time, ...
                'output_actions', {'SchedWaveTrig', 'cpoke_timer'; ...
                'DOut', cpoke_led+align_bit;...
                'SoundOut', nic_sound_id}, ...
                'input_to_statechange', {'Tup', 'current_state+1'});
            
        else
            
            sma = add_state(sma, 'name', 'CPOKE1', 'self_timer', min_time, ...
                'output_actions', {'SchedWaveTrig', 'cpoke_timer'; ...
                'DOut', cpoke_led+align_bit;}, ...
                'input_to_statechange', {'Tup', 'current_state+1'});
        end
        
        % Nose is in:  
        sma = add_state(sma, 'self_timer', 1000, ...
            'output_actions', {'DOut',          cpoke_led ; ...
                               'SchedWaveTrig', '-legal_cbreak'}, ...
            'input_to_statechange', {'Cout', 'current_state+1'; ...
                                     'cpoke_timer_In', nic_end_goto ; ...
                                     'Lin', poke_state ; ...
                                     'Rin', poke_state});
                                   
        % Nose just came out:                           
        sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'DOut',          cpoke_led ; ...
                               'SchedWaveTrig', 'legal_cbreak'}, ...
            'input_to_statechange', {'Cin', 'current_state-1'; ...
                                     'cpoke_timer_In', 'current_state+2'; ...
                                     'Tup', 'current_state+1'; ...
                                     'Lin', poke_state ; ...
                                     'Rin', poke_state});
                                          
        % Nose is out and neither cpoke_timer nor legal_cbreak timer have pinged yet:                           
        sma = add_state(sma, 'self_timer', 1000, ...
            'output_actions', {'DOut', cpoke_led}, ...
            'input_to_statechange', {'Cin', 'current_state-2'; ...
                                     'cpoke_timer_In',  'current_state+1'; ...
                                     'legal_cbreak_In', violation_goto; ...
                                     'Lin', poke_state ; ...
                                     'Rin', poke_state});
                                   
        % Nose is out and cpoke_timer has pinged but legal_cbreak hasn't pinged yet:                           
        sma = add_state(sma, 'self_timer', 1000, ...
            'output_actions', {'DOut', cpoke_led}, ...
            'input_to_statechange', {'Cin', nic_end_goto; ...
                                     'legal_cbreak_In', violation_goto; ...
                                     'Lin', poke_state ; ...
                                     'Rin', poke_state});
                                   
    end
    
     %% JEFF
      if nic_end_sound==0, nic_end_sound_id = 0;
      else                 nic_end_sound_id = SoundManagerSection(obj, 'get_sound_id', 'nic_end_sound');
      end;
    
        if WaitForCenterNoseOut==1, 
            wfcno_sound_id=nic_end_sound_id;
            c2sgap_sound_id=0;
            
        else
             c2sgap_sound_id=nic_end_sound_id;
            wfcno_sound_id=0;
        end;
    
    
    
%% nic_end_goto
   if PunishSidePokesinWFCO==1, side_poke_while_WFCO = 'punish_WFCO';
   else                         side_poke_while_WFCO = 'current_state';
   end;

   sma = add_state(sma, 'name', 'WAIT_FOR_CENTER_NOSE_OUT', ...
    'output_actions', {'SoundOut', -nic_sound_id},...
    'self_timer',0.0001,...
    'input_to_statechange',{'Tup','current_state+1'});

   sma = add_state(sma, ...
  'output_actions', {'SoundOut', wfcno_sound_id},...
    'input_to_statechange', {'Cout', 'center_2_side_gap', ...
                              'Lin',  side_poke_while_WFCO, ...
                              'Rin',  side_poke_while_WFCO});

                          
   pwfco_dur = SoundManagerSection(obj, 'get_sound_duration', 'WFCOPun');
   pwfco_id  = SoundManagerSection(obj, 'get_sound_id',       'WFCOPun');

                           
   sma = add_state(sma, 'name', 'PUNISH_WFCO', 'self_timer', pwfco_dur, ...
     'output_actions', {'SoundOut', pwfco_id}, ...
     'input_to_statechange', {'Tup', 'wait_for_center_nose_out', ...
                              'Cout', 'current_state+1'});
                            
   sma = add_state(sma, 'self_timer', pwfco_dur, ...
     'input_to_statechange', {'Tup', 'center_2_side_gap'});
                              
                            
                            

%% temp_violation
    % in the "one cpoke w/legal breaks" mode, we can choose whether to
    % respond to a violation by ending the trial or by a temp
    % violation that returns to wait_for_cpoke1

    if ~count_cpokes,
      v_dur = SoundManagerSection(obj, 'get_sound_duration', 'TempPun');
      v_id  = SoundManagerSection(obj, 'get_sound_id',       'TempPun');
      sma = add_scheduled_wave(sma, 'name', 'violation', 'preamble', min_time, 'sustain', v_dur, 'sound_trig', v_id);

      switch(value(TempPunChoice)),
        case 'reinit-able temp pun',
          % First turn off stimulus sounds and stimulus waves
          sma = add_state(sma, 'name', 'TEMP_VIOLATION', 'self_timer', min_time, ...
            'output_actions', {'SoundOut', -s1_id ; ...
                               'DOut',    cpoke_led ; ...
                               'SchedWaveTrig', waves_off'}, ...
            'input_to_statechange', {'Tup', 'current_state+1'});
          % Now turn off sound 2:                               
          sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'SoundOut', -s2_id ; ...
                               'DOut',    cpoke_led ; ...
                               'SchedWaveTrig', '-cpoke_timer'}, ...
            'input_to_statechange', {'Tup', 'current_state+1'});                        

          sma = PunishInterface(obj, 'add_sma_states', 'RTempPun', sma, ...
            'exitstate', 'wait_for_cpoke1');

        case 'non-reinit temp pun',

          % First turn off stimulus sounds and stimulus waves, and turn on violation wave (which turns on the violation sound):
          sma = add_state(sma, 'name', 'TEMP_VIOLATION', 'self_timer', min_time, ...
            'output_actions', {'SoundOut', -s1_id ; ...
                               'DOut',    cpoke_led ; ...
                               'SchedWaveTrig', waves_off'}, ...
            'input_to_statechange', {'Tup', 'current_state+1'; ...
                                     'Cin', 'current_state+3'});
          % Now turn off sound 2 (currently nose is OUT):                               
          sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'SoundOut', -s2_id ; ...
                               'DOut',    cpoke_led ; ...
                               'SchedWaveTrig', '-cpoke_timer'}, ...
            'input_to_statechange', {'Tup', 'current_state+1'; ...
                                     'Cin', 'current_state+3'});                        
          % Now turn on violation wave (currently nose is OUT):
          sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'DOut',    cpoke_led ; ...
                               'SchedWaveTrig', 'violation'}, ...
            'input_to_statechange', {'Tup', 'current_state+3'; ...
                                     'Cin', 'current_state+4'});                        
          % -- parallel states with nose IN:

          % Now turn off sound 2 (currently nose is IN):
          sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'SoundOut', -s2_id ; ...
                               'DOut',    cpoke_led ; ...
                               'SchedWaveTrig', '-cpoke_timer'}, ...
            'input_to_statechange', {'Tup',  'current_state+1'; ...
                                     'Cout', 'current_state-1'});                        
          % Now turn on violation wave (currently nose is IN):
          sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'DOut',    cpoke_led ; ...
                               'SchedWaveTrig', 'violation'}, ...
            'input_to_statechange', {'Tup',  'current_state+2'; ...
                                     'Cout', 'current_state+1'});                        

          % ---
      
          % Then just wait to clean go back (currently nose is OUT):
          sma = add_state(sma, 'output_actions', {'DOut', cpoke_led}, ...
            'input_to_statechange', {'violation_Out', 'wait_for_cpoke1'; ...
                                     'Cin',          'current_state+1'});
          % Then just wait to clean go back (currently nose is IN):
          sma = add_state(sma, 'output_actions', {'DOut', cpoke_led}, ...
            'input_to_statechange', {'violation_Out', 'play_stimulus_sound'; ...
                                     'Cout',         'current_state-1'});
      
          % -- END of CASE NON-REINIT TEMP PUN                                   
     end;
   end;
    

%% free center poke grazing states for sounds

    sma = add_state(sma, 'name', 'center_in_with_sounds', ...
        'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', waves_off}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', waves_on}, ...
        'input_to_statechange', {'Tup', 'nose_in_center'});
    
    % nose in: sound is playing, wait for him to Cout
    sma = add_state(sma, 'name', 'nose_in_center', ...
        'output_actions', {'DOut', cpoke_led}, ...
        'input_to_statechange', {'Cout', 'wait_for_poke'});
    
    % wait for next poke:
    % if cpoke, go back to play sounds
    % if side poke, go to appropriate response
    sma = add_state(sma, 'name', 'wait_for_poke', ...
        'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', waves_off; ...
                           'SoundOut', -s1_id;}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, ...
        'input_to_statechange', {'Cin', 'center_in_with_sounds+1'; ...
                                 'Lin', LeftResponseState; ...
                                 'Rin', RightResponseState;});
        
    
%% center to side gap
    % these states are only reached when n_center_pokes == 1
    if PunishC2SCenterPokes == 1, center_poke_state = 'c2s_pun';
    else                          center_poke_state = 'current_state';
    end
    
    if PunishC2SSidePokes == 1, 
        lside_poke_state = 'c2s_pun';
        rside_poke_state = 'c2s_pun';
    else
        lside_poke_state = 'current_state+1';
        rside_poke_state = 'current_state+2';
    end;
    
    if C2SGap_2 == 0, 
        after_c2sgap_state    = 'wait_for_spoke';
        c2sgap_left_response  = LeftResponseState;
        c2sgap_right_response = RightResponseState;
    else
        after_c2sgap_state    = 'center_2_side_gap_2';
        c2sgap_left_response  = 'center_2_side_gap_2+4';
        c2sgap_right_response = 'center_2_side_gap_2+5';
    end
    
    sma = add_scheduled_wave(sma, 'name', 'c2s_gap', 'preamble', max(min_time, C2SGap));
    
   
    
    
    sma = add_state(sma, 'name', 'center_2_side_gap',... 
        'self_timer', 0.001,...
        'input_to_statechange', {'Tup', 'current_state+1'});
    
    sma = add_state(sma, 'name', 'center_2_side_gap_wait', ...
        'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', '-c2s_gap'; ...
                           'SoundOut',      c2sgap_sound_id}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    if strcmp(stim_timing, 'During Cpoke Only'),  
        % if we want the sounds to be turned off as soon as the center poke states are done,
        sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'SchedWaveTrig', waves_off; ...
                               'SoundOut', -s1_id;}, ...
            'input_to_statechange', {'Tup', 'current_state+1'});
        
    end;
    sma = add_state(sma,  ...
        'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', 'c2s_gap';...
                           'SoundOut', -nic_sound_id; }, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma,  ...
        'input_to_statechange', {'c2s_gap_In', after_c2sgap_state; ...
                                 'Cin', center_poke_state; ...
                                 'Lin', lside_poke_state; ...
                                 'Rin', rside_poke_state; ...
                                 });
    sma = add_state(sma, ...  % LEFT
        'input_to_statechange', {'c2s_gap_In', c2sgap_left_response; ...
                                 'Lout', 'current_state-1'; ...
                                 });
    sma = add_state(sma, ...  % RIGHT
        'input_to_statechange', {'c2s_gap_In', c2sgap_right_response; ...
                                 'Rout', 'current_state-2'; ...
                                 });
                             
    % c2s_2, where nothing is punished (or punishable)
    sma = add_scheduled_wave(sma, 'name', 'c2s_gap_2', 'preamble', max(min_time, C2SGap_2));
    
    sma = add_state(sma, 'name', 'center_2_side_gap_2', ...
        'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', '-c2s_gap_2'}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', 'c2s_gap_2'}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, ...
        'input_to_statechange', {'c2s_gap_2_In', 'wait_for_spoke'; ...
                                 'Lin', 'current_state+1'; ...
                                 'Rin', 'current_state+2'});
    sma = add_state(sma, ... % LEFT
        'input_to_statechange', {'c2s_gap_2_In', LeftResponseState; ...
                                 'Lout', 'current_state-1'; ...
                                });
    sma = add_state(sma, ... % RIGHT
        'input_to_statechange', {'c2s_gap_2_In', RightResponseState; ...
                                 'Lout', 'current_state-2'; ...
                                });
        
    % c2s punishment states: first turn off the stimulus sound
    sma = add_state(sma, 'name', 'c2s_pun', ...
        'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s1_id}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s2_id; ...
                           'SchedWaveTrig', waves_off}, ...
        'input_to_statechange', {'Tup', 'c2s_pun2'});
    % then add punishment states

    sma = PunishInterface(obj, 'add_sma_states', 'C2SPun', sma, ...
        'name', 'c2s_pun2', ...
        'exitstate', AfterC2SPunishment);

%% side poke

if flash_side_lights==1

    
    sld=DistribInterface(obj, 'get_current_sample', 'side_lights_dur');
    
    sma = add_state(sma, 'name', 'wait_for_spoke', ...
        'self_timer', sld, ...
        'output_actions', {'DOut', side_led}, ...
        'input_to_statechange', {'Lin', LeftResponseState; ...
                                 'Rin', RightResponseState; ...
                                 'Tup', 'current_state+1'; ...
                                });
    if LEDMaskDur > 0                        
    sma = add_state(sma,  ...
        'self_timer', LEDMaskDelay, ...
        'input_to_statechange', {'Lin', LeftResponseState; ...
                                 'Rin', RightResponseState; ...
                                 'Tup', 'current_state+1'; ...
                                });                        
                            
    sma = add_state(sma,  ...
        'self_timer', LEDMaskDur, ...
        'output_actions', {'DOut', left1led+right1led}, ...
        'input_to_statechange', {'Lin', LeftResponseState; ...
                                 'Rin', RightResponseState; ...
                                 'Tup', 'current_state+1'; ...
                                });                    
    end
                            
   left_over_grace= wait_for_spoke_grace-LEDMaskDur-LEDMaskDelay-sld;
   if left_over_grace<min_time
       left_over_grace=min_time;
   end
                            
     sma = add_state(sma,  ...
        'self_timer', left_over_grace, ...
        'input_to_statechange', {'Lin', LeftResponseState; ...
                                 'Rin', RightResponseState; ...
                                 'Tup', 'violation_state'; ...
                                });                            
                       
else
    sma = add_state(sma, 'name', 'wait_for_spoke', ...
        'self_timer', wait_for_spoke_grace, ...
        'output_actions', {'DOut', side_led}, ...
        'input_to_statechange', {'Lin', LeftResponseState; ...
                                 'Rin', RightResponseState; ...
                                 'Tup', 'violation_state'; ...
                                });
end                        
                            
%% error states
    sma = add_state(sma, 'name', 'error_state1', ...
        'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s1_id; ...
                           'SchedWaveTrig', waves_off;}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s2_id}, ...
        'input_to_statechange', {'Tup', 'error_state'});
    
    sma = PunishInterface(obj, 'add_sma_states', 'ErrorPun', sma, ...
        'name', 'error_state', ...
        'exitstate', 'clean_up_state');

%% temporary_punishment      
    pundur = PunishInterface(SameDifferent, 'get', 'TimeOutPun', 'Duration');
    sma = add_scheduled_wave(sma, 'name', 'temporary_punishment_lights_wave', 'preamble', min_time, ...
      'sustain', pundur-min_time, 'DOut', side_led);
    
    if LEDtemp_pun==1,
      sma = add_state(sma, 'name', 'temporary_punishment', 'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', 'temporary_punishment_lights_wave'}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
      
      sma = PunishInterface(obj, 'add_sma_states', 'TimeOutPun', sma, ...
        'name', 'temporary_punishment2', ...
        'exitstate', 'wait_for_spoke');
    else
      sma = PunishInterface(obj, 'add_sma_states', 'TimeOutPun', sma, ...
        'name', 'temporary_punishment', ...
        'exitstate', 'wait_for_spoke');      
    end;
    
%% violation states
    violation_snd_id = SoundManagerSection(obj, 'get_sound_id', 'TimeOutPun_OngoingSnd');
    sma = add_state(sma, 'name', 'violation_state', ...
        'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s1_id; ...
                           'SchedWaveTrig', waves_off}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s2_id;}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -nic_sound_id;}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', violation_punishment, ...
        'output_actions', {'SoundOut', violation_snd_id}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -violation_snd_id}, ...
        'input_to_statechange', {'Tup', 'clean_up_state'});
    

%% reward states
    sma = add_scheduled_wave(sma, 'name', 'stop_stimulus_sound', 'preamble', max(from_answer_poke, min_time));

    sma = add_scheduled_wave(sma, 'name', 'leds_during_reward', 'preamble', min_time, ...
      'sustain', max(min_time, LEDRewardTime), 'DOut', side_led);
    if LEDRewardTime <= 0, swaves_to_add = '';
    else                   swaves_to_add = '+leds_during_reward';
    end;
    
    if strcmp(stim_timing, 'From Answer Poke'), % if we're in FromAnswerPoke, start the timer now
        sma = add_state(sma, 'name', 'left_reward', 'self_timer', min_time, ...
            'output_actions', {'SchedWaveTrig', ['direct_reward+stop_stimulus_sound' swaves_to_add];...
            'SoundOut',RewardSound}, ...
            'input_to_statechange', {'Tup', 'hit_state';'stop_stimulus_sound_In', 'current_state+1'});
        sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'SoundOut', -s1_id}, ...
            'input_to_statechange', {'Tup', 'current_state+1'});
        sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'SoundOut', -s2_id; ...
                               'SchedWaveTrig', waves_off}, ...
            'input_to_statechange', {'Tup', 'hit_state'});
    else % else if sounds are timed from stim start or by cpoke
        sma = add_state(sma, 'name', 'left_reward', 'self_timer', min_time, ...
            'output_actions', {'SoundOut',RewardSound;'SchedWaveTrig', ['direct_reward' swaves_to_add]}, ...
            'input_to_statechange', {'Tup', 'hit_state'});        
    end;
    
    
    if strcmp(stim_timing, 'From Answer Poke'), % if we're in FromAnswerPoke, start the timer now
        sma = add_state(sma, 'name', 'right_reward', 'self_timer', min_time, ...
            'output_actions', {'SoundOut',RewardSound;'SchedWaveTrig', ['direct_reward+stop_stimulus_sound' swaves_to_add]}, ...
            'input_to_statechange', {'Tup', 'hit_state'; ...
                                     'stop_stimulus_sound_In', 'current_state+1'});
        sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'SoundOut', -s1_id}, ...
            'input_to_statechange', {'Tup', 'current_state+1'});
        sma = add_state(sma, 'self_timer', min_time, ...
            'output_actions', {'SoundOut', -s2_id; ...
                               'SchedWaveTrig', waves_off}, ...
            'input_to_statechange', {'Tup', 'hit_state'});
    else % else if sounds are timed from stim start or by cpoke
        sma = add_state(sma, 'name', 'right_reward', 'self_timer', min_time, ...
            'output_actions', {'SoundOut',RewardSound;'SchedWaveTrig', ['direct_reward' swaves_to_add]}, ...
            'input_to_statechange', {'Tup', 'hit_state'});
    end;
            
    
    
%     
%     
%     
%     sma = add_scheduled_wave(sma, 'name', 'left_water_valve',  'preamble', max(LeftWValveTime*this_wtr_mul, min_time));
%     sma = add_scheduled_wave(sma, 'name', 'right_water_valve', 'preamble', max(RightWValveTime*this_wtr_mul, min_time));
%     
%     % LEFT water delivery
%     if StimTimeStart == 0,
%         sma = add_state(sma, 'name', 'left_reward', ...
%             'self_timer', LeftWValveTime*this_wtr_mul, ...
%             'output_actions', {'DOut', left1water}, ...
%             'input_to_statechange', {'Tup', 'hit_state'});
%     else % if we're in From Answer Poke, start the timer now
%         sma = add_state(sma, 'name', 'left_reward', ...
%             'output_actions', {'SchedWaveTrig', 'stop_stimulus_sound+left_water_valve'; ...
%                                'DOut', left1water}, ...
%             'input_to_statechange', {'left_water_valve_In', 'hit_state'; ...
%                                      'stop_stimulus_sound_In', 'current_state+1'});
%         sma = add_state(sma, 'self_timer', min_time, ...
%             'output_actions', {'DOut', left1water; ...
%                                'SoundOut', -s1_id;}, ...
%             'input_to_statechange', {'Tup', 'current_state+1'; ...
%                                      'left_water_valve_In', 'hit_state+1'});
%         sma = add_state(sma, ...
%             'output_actions', {'DOut', left1water; ...
%                                'SoundOut', -s2_id;}, ...
%             'input_to_statechange', {'left_water_valve_In', 'hit_state+1'});
%     end;
%     
%     % RIGHT water delivery
%     if StimTimeStart == 0,
%         sma = add_state(sma, 'name', 'right_reward', ...
%             'self_timer', RightWValveTime*this_wtr_mul, ...
%             'output_actions', {'DOut', right1water}, ...
%             'input_to_statechange', {'Tup', 'hit_state'});
%     else % if we're in From Answer Poke, start the timer now
%         sma = add_state(sma, 'name', 'right_reward', ...
%             'output_actions', {'SchedWaveTrig', 'stop_stimulus_sound+right_water_valve'; ...
%                                'DOut', right1water}, ...
%             'input_to_statechange', {'right_water_valve_In', 'hit_state'; ...
%                                      'stop_stimulus_sound_In', 'current_state+1'});
%         sma = add_state(sma, 'self_timer', min_time, ...
%             'output_actions', {'DOut', right1water; ...
%                                'SoundOut', -s1_id;}, ...
%             'input_to_statechange', {'Tup', 'current_state+1'; ...
%                                      'right_water_valve_In', 'hit_state+1'});
%         sma = add_state(sma, ...
%             'output_actions', {'DOut', right1water; ...
%                                'SoundOut', -s2_id;}, ...
%             'input_to_statechange', {'right_water_valve_In', 'hit_state+1'});
%     end;
%     



%% hit states
    
        

    if SoftDrinkTime == 1, 
      post_hit_state_in  = 'soft_drink_state';
      post_hit_state_out = 'soft_drink_state+1';
    else
      post_hit_state_in  = 'drink_state';
      post_hit_state_out = 'drink_state';
    end;
    
    if L_baited==0 && SidesSection(obj, 'get_current_side')=='l'
        post_hit_state_in='clean_up_state';
        post_hit_state_out='clean_up_state';
    end
        

    if R_baited==0 && SidesSection(obj, 'get_current_side')=='r'
        post_hit_state_in='clean_up_state';
        post_hit_state_out='clean_up_state';
    end

    
    switch SidesSection(obj, 'get_current_side'),
      case 'l', InAction = 'Lin'; OutAction = 'Lout'; OtherSidePokeIn = 'Rin';
      case 'r', InAction = 'Rin'; OutAction = 'Rout'; OtherSidePokeIn = 'Lin';
      otherwise
        error('SAMEDIFFERENT:Invalid', 'Don''t know how to deal with current_side "%s"', SidesSection(obj, 'get_current_side'));
    end;


    sma = add_scheduled_wave(sma, 'name', 'hitstatetimer', 'preamble', DrinkTime);
   
    % allows rat to continue drinking for the time allotted in
    % PenaltySection_DrinkTime
    % First we just trigger the hit_state_timer:
    sma = add_state(sma, 'name', 'hit_state', 'self_timer', min_time, ...
      'output_actions', {'SchedWaveTrig', 'hitstatetimer'}, ...
      'input_to_statechange', {'Tup', 'current_state+2'});
    sma = add_state(sma, 'self_timer', min_time, ...
      'output_actions', {'SchedWaveTrig', 'hitstatetimer'}, ...
      'input_to_statechange', {'Tup', 'current_state+3'});
    % Then we toggle back and forth between being in or out of the drink
    % port:
    sma = add_state(sma, 'input_to_statechange', ...
      {'hitstatetimer_In', post_hit_state_in ; ...
      'stop_stimulus_sound_In', 'current_state+2' ; ...
      OutAction, 'current_state+1'});
    sma = add_state(sma, 'input_to_statechange', ...
      {'hitstatetimer_In', post_hit_state_out; ...
      'stop_stimulus_sound_In', 'current_state+2' ; ...
      InAction, 'current_state-1'});   
    % Now duplicates to turn stimulus sound off:
    sma = add_state(sma, 'self_timer', min_time, 'input_to_statechange', ...
      {'Tup', 'current_state-2' ; ...
      'hitstatetimer_In', post_hit_state_in ; ...
      OutAction, 'current_state-1'}, ...
      'output_actions', {'SoundOut' -s2_id; ...
                         'SchedWaveTrig', waves_off});
    sma = add_state(sma, 'self_timer', min_time, 'input_to_statechange', ...
      {'Tup', 'current_state-2' ; ...
      'hitstatetimer_In', post_hit_state_out ; ...
      InAction, 'current_state-3'}, ...
      'output_actions', {'SoundOut' -s2_id; ...
                         'SchedWaveTrig', waves_off});
    
                                     
    % timed drinking states:
    if WarnDur > 0,                     post_drink_state = 'warning';
    elseif WarnDur==0 && DangerDur > 0, post_drink_state = 'danger';
    elseif WarnDur==0 && DangerDur==0,  post_drink_state = 'clean_up_state';
    end;
    if DangerDur > 0,                   post_warning_state = 'danger';
    else                                post_warning_state = 'clean_up_state';
    end;   
    
    warn_id   = SoundManagerSection(obj, 'get_sound_id', 'WarningSound');
    danger_id = SoundManagerSection(obj, 'get_sound_id', 'DangerSound');

    sma = add_state(sma, 'name', 'DRINK_STATE', 'self_timer', min_time, ...
        'input_to_statechange', {'Tup', post_drink_state});
    sma = add_state(sma, 'name', 'WARNING', 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s2_id; ...
                           'SchedWaveTrig', waves_off}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', WarnDur, ...
        'input_to_statechange', {'Tup', post_warning_state}, ...
        'output_actions', {'SoundOut', warn_id});
    sma = add_state(sma, 'name', 'DANGER',  'self_timer', DangerDur, ...
        'output_actions', {'SoundOut', danger_id}, ...
        'input_to_statechange', {'Tup', 'clean_up_state', ...
                                 'Lin', 'current_state+1', ...
                                 'Cin', 'current_state+1', ...
                                 'Rin', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -danger_id}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = PunishInterface(obj, 'add_sma_states', 'PostDrinkPun', sma, 'name', 'mypun', ...
        'exitstate', 'danger');    
    
    
%% soft drink states
    if WD_in_sdt==1, post_softdt = post_drink_state;
    else             post_softdt = 'clean_up_state';
    end;    
    
    
    if SoftDrinkCap > DrinkTime, softdrinktimer = SoftDrinkCap-DrinkTime+min_time;
    else                         softdrinktimer = min_time;
    end;
    
    sma = add_scheduled_wave(sma, 'name', 'softdrinktimer', 'preamble', softdrinktimer);
    sma = add_scheduled_wave(sma, 'name', 'sd_grace',       'preamble', max(SoftDrinkGrace, 3*min_time));

    % Entry to soft_drink_state_in:
    sma = add_state(sma, 'name', 'SOFT_DRINK_STATE', ...
        'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', 'softdrinktimer'}, ...
        'input_to_statechange', {'Tup', 'current_state+2'});

    % Entry to soft_drink_state_out:
    sma = add_state(sma, ...
        'self_timer', min_time, ...
        'output_actions', {'SchedWaveTrig', 'softdrinktimer'}, ...
        'input_to_statechange', {'Tup', 'current_state+2'});

    sma = add_state(sma, 'output_actions', {'SchedWaveTrig', '-sd_grace'}, ...
      'input_to_statechange', {'softdrinktimer_In',      post_drink_state, ...
                               OutAction,                'current_state+1', ...
                               'Cin',                    post_softdt, ...
                               OtherSidePokeIn,          post_softdt, ... 
                               'stop_stimulus_sound_In', 'current_state+3', ...
                               });
    sma = add_state(sma, 'self_timer', min_time, 'output_actions', {'SchedWaveTrig', 'sd_grace'}, ...
      'input_to_statechange', {'Tup',                    'current_state+1' ; ...
                               'stop_stimulus_sound_In', 'current_state+3'});     
    sma = add_state(sma, ...
      'input_to_statechange', {'softdrinktimer_In', post_drink_state, ...     
                               InAction,            'current_state-2', ...
                               'Cin',               post_softdt, ...
                               OtherSidePokeIn,     post_softdt, ...         
                               'sd_grace_In',       post_softdt, ... 
                               'stop_stimulus_sound_In', 'current_state+2'});

    % Duplication of above two states to deal with stop_stimulus_sound_In:
    sma = add_state(sma, 'self_timer', min_time, 'output_actions', {'SoundOut', -s2_id; ...
                                                                    'SchedWaveTrig', waves_off}, ...
      'input_to_statechange', {'softdrinktimer_In',      post_drink_state, ...
                               OutAction,                'current_state-2', ...
                               'Cin',                    post_softdt, ...
                               OtherSidePokeIn,          post_softdt, ... 
                               'Tup',                    'current_state-3', ...
                               });
    sma = add_state(sma, 'self_timer', min_time, 'output_actions', {'SoundOut', -s2_id; ...
                                                                    'SchedWaveTrig', waves_off}, ...
      'input_to_statechange', {'softdrinktimer_In', post_drink_state, ...     
                               InAction,            'current_state-4', ...
                               'Cin',               post_softdt, ...
                               OtherSidePokeIn,     post_softdt, ... 
                               'sd_grace_In',       post_softdt, ...
                               'Tup',               'current_state-2'});

                             
     
%% clean up                        
      
    % clean up any remaining sounds still playing, then jump to next
    % trial
    sma = add_state(sma, 'name', 'clean_up_state', ...
        'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s1_id; ...
                           'SchedWaveTrig', waves_off}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    sma = add_state(sma, 'self_timer', min_time, ...
        'output_actions', {'SoundOut', -s2_id}, ...
        'input_to_statechange', {'Tup', 'current_state+1'});
    
    sma = add_state(sma,  ...
        'self_timer', min_time, ...
        'output_actions', {'SoundOut', -danger_id}, ...
        'input_to_statechange', {'Tup', 'check_next_trial_ready'});    
                            
    % modify the state matrix to give light flashes
    sma = StimulatorSection(obj,'prepare_next_trial',sma);
    % add states to the 'prepare_next_trial' states:
    dispatcher('send_assembler', sma, {'hit_state', 'error_state', 'violation_state'});
    
%% reinit
  case 'reinit',

    % Delete all SoloParamHandles who belong to this object and whose
    % fullname starts with the name of this mfile:
    delete_sphandle('owner', ['^@' class(obj) '$'], ...
      'fullname', ['^' mfilename]);

    % Reinitialise at the original GUI position and figure:
    feval(mfilename, obj, 'init');
    
end;